@@ -103,7 +103,12 @@ groupwise_store_construct (CamelService *service, CamelSession *session,
 				     _("Host or user not available in url"));
 	}
 
-	groupwise_store->list_loaded = FALSE;
+	/*XXX: The number 3 assigned to the list_loaded variable denotes
+	 * the number of times the get_folder_info is called during startup.
+	 * We are just trying to minimize the call.
+	 * This is a dirty hack. But it *WORKS*
+	 */
+	groupwise_store->list_loaded = 3;
 	
 	/*storage path*/
 	priv->storage_path = camel_session_get_storage_path (session, service, ex);
@@ -183,6 +188,7 @@ groupwise_auth_loop (CamelService *service, CamelException *ex)
 	else 
 		uri = g_strconcat ("http://", priv->server_name, ":", priv->port, "/soap", NULL);
 	service->url->passwd = NULL;
+	
 
 	while (!authenticated) {
 		if (errbuf) {
@@ -192,6 +198,7 @@ groupwise_auth_loop (CamelService *service, CamelException *ex)
 			service->url->passwd = NULL;
 		}
 		
+	
 		if (!service->url->passwd && !(store->flags & CAMEL_STORE_PROXY)) {
 			char *prompt;
 			
@@ -209,7 +216,7 @@ groupwise_auth_loop (CamelService *service, CamelException *ex)
 			
 			if (!service->url->passwd) {
 				camel_exception_set (ex, CAMEL_EXCEPTION_USER_CANCEL,
-						     _("You didn't enter a password."));
+						     _("You did not enter a password."));
 				return FALSE;
 			}
 		}
@@ -222,7 +229,7 @@ groupwise_auth_loop (CamelService *service, CamelException *ex)
 		}
 		if (!E_IS_GW_CONNECTION(priv->cnc)) {
 			errbuf = g_strdup_printf (_("Unable to authenticate "
-					    "to GroupWise server."));
+					    "to GroupWise server. "));
 						  
 			camel_exception_clear (ex);
 		} else 
@@ -292,6 +299,7 @@ groupwise_connect (CamelService *service, CamelException *ex)
 		return FALSE;
 	}
 	
+	
 	service->status = CAMEL_SERVICE_CONNECTED;
 	((CamelOfflineStore *) store)->state = CAMEL_OFFLINE_STORE_NETWORK_AVAIL;
 
@@ -307,14 +315,6 @@ groupwise_connect (CamelService *service, CamelException *ex)
 	camel_groupwise_store_summary_namespace_set (store->summary, ns);
 
 	if (camel_store_summary_count ((CamelStoreSummary *)store->summary) == 0) {
-		/*XXX: Have to sync up folders here*/
-		groupwise_folders_sync (store, ex);
-		if (camel_exception_is_set (ex)) {
-			camel_store_summary_save ((CamelStoreSummary *) store->summary);
-			CAMEL_SERVICE_UNLOCK (service, connect_lock);
-			camel_service_disconnect (service, TRUE, NULL);
-			return FALSE;
-		}
 		/*Settting the refresh stamp to the current time*/
 		store->refresh_stamp = time (0);
 	}
@@ -458,7 +458,6 @@ groupwise_build_folder_info(CamelGroupwiseStore *gw_store, const char *parent_na
 		name = fi->full_name;
 	else
 		name++;
-
 	if (!strcmp (folder_name, "Sent Items"))
 		fi->flags |= CAMEL_FOLDER_TYPE_SENT;
 	else if (!strcmp (folder_name, "Mailbox"))
@@ -555,7 +554,7 @@ groupwise_get_folder (CamelStore *store, const char *folder_name, guint32 flags,
 	char *container_id, *folder_dir, *storage_path;
 	EGwConnectionStatus status;
 	GList *list = NULL;
-	gboolean done = FALSE;
+	gboolean done = FALSE, all_ok = TRUE;
 	const char *position = E_GW_CURSOR_POSITION_END; 
 	int count = 0, cursor, summary_count = 0;
 	CamelStoreInfo *si = NULL;
@@ -604,7 +603,7 @@ groupwise_get_folder (CamelStore *store, const char *folder_name, guint32 flags,
 		return NULL;
 	}
 	g_free (folder_dir);
-
+	
 	si = camel_store_summary_path ((CamelStoreSummary *)gw_store->summary, folder_name);
 	if (si) {
 		total = si->total;
@@ -618,7 +617,7 @@ groupwise_get_folder (CamelStore *store, const char *folder_name, guint32 flags,
 		d(g_print ("\n\n** %s **: No summary as yet : using get cursor request\n\n", folder->name);)
 
 		status = e_gw_connection_create_cursor (priv->cnc, container_id, 
-				"peek id recipient attachments distribution subject status options priority startDate created delivered size",
+				"peek id recipient attachments distribution subject status options priority startDate created delivered size hasAttachment",
 				NULL,
 				&cursor);
 		if (status != E_GW_CONNECTION_STATUS_OK) {
@@ -635,15 +634,126 @@ groupwise_get_folder (CamelStore *store, const char *folder_name, guint32 flags,
 							      cursor, FALSE, 
 							      CURSOR_ITEM_LIMIT, position, &list);
 			if (status != E_GW_CONNECTION_STATUS_OK) {
+				all_ok = FALSE;
+				break;
+				/*
 				CAMEL_SERVICE_UNLOCK (gw_store, connect_lock);
 				e_gw_connection_destroy_cursor (priv->cnc, container_id, cursor);
-				camel_folder_summary_clear (folder->summary);
+				//camel_folder_summary_clear (folder->summary);
 				camel_folder_summary_save (folder->summary);
 				camel_exception_set (ex, CAMEL_EXCEPTION_SERVICE_INVALID, _("Authentication failed"));
 				camel_operation_end (NULL);
 				camel_object_unref (folder);
 				g_free (container_id);
-				return NULL;
+				return NULL;*/
+			}
+			
+			count += g_list_length (list);
+		
+			if (total > 0)
+				camel_operation_progress (NULL, (100*count)/total);
+			gw_update_summary (folder, list,  ex);
+			
+			if (!list)
+				done = TRUE;
+			g_list_foreach (list, (GFunc)g_object_unref, NULL);
+			g_list_free (list);
+			list = NULL;
+			position = E_GW_CURSOR_POSITION_CURRENT;
+      		}
+
+		e_gw_connection_destroy_cursor (priv->cnc, container_id, cursor);
+
+		camel_operation_end (NULL);
+	} 
+	if (done && all_ok) {
+		if (summary->time_string)
+			g_free (summary->time_string);
+		summary->time_string = g_strdup (e_gw_connection_get_server_time (priv->cnc));
+	}
+
+	camel_folder_summary_save (folder->summary);
+
+	gw_store->current_folder = folder;
+	camel_object_ref (folder);
+
+	g_free (container_id);
+	CAMEL_SERVICE_UNLOCK (gw_store, connect_lock);
+
+	return folder;
+}
+
+void 
+gw_store_reload_folder (CamelGroupwiseStore *gw_store, CamelFolder *folder, guint32 flags, CamelException *ex)
+{
+	CamelGroupwiseStorePrivate *priv = gw_store->priv;
+	CamelGroupwiseSummary *summary;
+	char *container_id;
+	EGwConnectionStatus status;
+	GList *list = NULL;
+	gboolean done = FALSE;
+	const char *position = E_GW_CURSOR_POSITION_END; 
+	int count = 0, cursor, summary_count = 0;
+	CamelStoreInfo *si = NULL;
+	guint total;
+	
+	camel_exception_clear (ex);
+
+	CAMEL_SERVICE_LOCK (gw_store, connect_lock);
+
+	if (!camel_groupwise_store_connected (gw_store, ex)) {
+		CAMEL_SERVICE_UNLOCK (gw_store, connect_lock);
+		return;
+	}
+	
+	if (!E_IS_GW_CONNECTION( priv->cnc)) {
+		if (!groupwise_connect (CAMEL_SERVICE((CamelStore*)gw_store), ex)) {
+			camel_exception_set (ex, CAMEL_EXCEPTION_SERVICE_CANT_AUTHENTICATE, _("Authentication failed"));
+			CAMEL_SERVICE_UNLOCK (gw_store, connect_lock);
+			return;
+		}
+	}
+	
+	container_id = 	g_strdup (g_hash_table_lookup (priv->name_hash, folder->full_name));
+
+	si = camel_store_summary_path ((CamelStoreSummary *)gw_store->summary, folder->name);
+	if (si) {
+		total = si->total;
+		camel_store_summary_info_free ((CamelStoreSummary *)(gw_store)->summary, si);
+	}
+
+	summary = (CamelGroupwiseSummary *) folder->summary;
+	camel_folder_summary_clear (folder->summary);
+	camel_folder_summary_save (folder->summary);
+
+	summary_count = camel_folder_summary_count (folder->summary);
+	if(!summary_count || !summary->time_string) {
+		d(g_print ("\n\n** %s **: Summary missing???? Reloading summary....\n\n", folder->name);)
+
+		status = e_gw_connection_create_cursor (priv->cnc, container_id, 
+				"peek id recipient attachments distribution subject status options priority startDate created delivered size",
+				NULL,
+				&cursor);
+		if (status != E_GW_CONNECTION_STATUS_OK) {
+			CAMEL_SERVICE_UNLOCK (gw_store, connect_lock);
+			g_free (container_id);
+			return;
+		}
+
+		camel_operation_start (NULL, _("Fetching summary information for new messages in %s"), folder->name);
+
+		while (!done) {
+			status = e_gw_connection_read_cursor (priv->cnc, container_id, 
+							      cursor, FALSE, 
+							      CURSOR_ITEM_LIMIT, position, &list);
+			if (status != E_GW_CONNECTION_STATUS_OK) {
+				CAMEL_SERVICE_UNLOCK (gw_store, connect_lock);
+				e_gw_connection_destroy_cursor (priv->cnc, container_id, cursor);
+				camel_folder_summary_save (folder->summary);
+				camel_exception_set (ex, CAMEL_EXCEPTION_SERVICE_INVALID, _("Authentication failed"));
+				camel_operation_end (NULL);
+				g_free (container_id);
+				return;
 			}
 			
 			count += g_list_length (list);
@@ -674,12 +784,10 @@ groupwise_get_folder (CamelStore *store, const char *folder_name, guint32 flags,
 	camel_folder_summary_save (folder->summary);
 
 	gw_store->current_folder = folder;
-	camel_object_ref (folder);
-
+	
 	g_free (container_id);
 	CAMEL_SERVICE_UNLOCK (gw_store, connect_lock);
-
-	return folder;
+	return;
 }
 
 CamelFolderInfo *
@@ -766,7 +874,7 @@ convert_to_folder_info (CamelGroupwiseStore *store, EGwContainer *container, con
 	if (e_gw_container_get_is_shared_by_me (container))
 		fi->flags |= CAMEL_FOLDER_SHARED_BY_ME;
 
-/*	if (type == E_GW_CONTAINER_TYPE_INBOX) {
+	/*if (type == E_GW_CONTAINER_TYPE_INBOX) {
 		fi->total = -1;
 		fi->unread = -1;
 	} else	if (type == E_GW_CONTAINER_TYPE_TRASH) {*/
@@ -810,6 +918,13 @@ groupwise_folders_sync (CamelGroupwiseStore *store, CamelException *ex)
 	CamelStoreInfo *si = NULL;
 	int count, i;
 
+	if (!priv->cnc && ((CamelOfflineStore *) store)->state == CAMEL_OFFLINE_STORE_NETWORK_AVAIL) {
+		if (((CamelService *)store)->status == CAMEL_SERVICE_DISCONNECTED){
+			((CamelService *)store)->status = CAMEL_SERVICE_CONNECTING;
+			groupwise_connect ((CamelService *)store, ex);
+		}
+	}
+
 	status = e_gw_connection_get_container_list (priv->cnc, "folders", &folder_list);
 	if (status != E_GW_CONNECTION_STATUS_OK) {
 		g_warning ("Could not get folder list..\n");
@@ -870,7 +985,6 @@ groupwise_folders_sync (CamelGroupwiseStore *store, CamelException *ex)
 				camel_folder_info_free (info);
 				info = NULL;
 			}
-
 		}
 	}
 	
@@ -1017,36 +1131,30 @@ groupwise_get_folder_info (CamelStore *store, const char *top, guint32 flags, Ca
 	 * Thanks to Michael, for his cached folders implementation in IMAP
 	 * is used as is here.
 	 */
-	if ((groupwise_store->list_loaded == TRUE) && camel_store_summary_count((CamelStoreSummary *)groupwise_store->summary) > 0) {
-		/*Load from cache*/
-		/*
-		time_t now;
-		int ref;
-
-		now = time (0);
-		ref = now > groupwise_store->refresh_stamp+60*60*1;
-		if (ref) {
-			CAMEL_SERVICE_LOCK (store, connect_lock);
-			ref = now > groupwise_store->refresh_stamp+60*60*1;
-			if (ref) {
-				struct _store_refresh_msg *m;
-				groupwise_store->refresh_stamp = now;
-				m = camel_session_thread_msg_new (((CamelService *)store)->session, &store_refresh_ops, sizeof(*m));
-				m->store = store;
-				camel_object_ref (store);
-				camel_exception_init (&m->ex);
-				camel_session_thread_queue (((CamelService *)store)->session, &m->msg, 0);
-			}
+	if (camel_store_summary_count ((CamelStoreSummary *)groupwise_store->summary) == 0) {
+		CAMEL_SERVICE_LOCK (store, connect_lock);
+		if (groupwise_store->list_loaded == 3) {
+			groupwise_folders_sync (groupwise_store, ex);
+			groupwise_store->list_loaded -= 1;
+		}
+		if (camel_exception_is_set (ex)) {
+			camel_store_summary_save ((CamelStoreSummary *) groupwise_store->summary);
 			CAMEL_SERVICE_UNLOCK (store, connect_lock);
-		}*/
-		if (groupwise_store->current_folder)
-			CAMEL_FOLDER_CLASS (CAMEL_OBJECT_GET_CLASS (groupwise_store->current_folder))->sync(groupwise_store->current_folder, FALSE, ex);
-		groupwise_folders_sync (groupwise_store, ex);
+			return NULL;
+		}
+		CAMEL_SERVICE_UNLOCK (store, connect_lock);
+		camel_store_summary_save ((CamelStoreSummary *)groupwise_store->summary);
+		goto end_r;
+	}
+
+	if ((camel_store_summary_count((CamelStoreSummary *)groupwise_store->summary) > 0) && (groupwise_store->list_loaded > 1)) {
+		/*Load from cache*/
+		groupwise_store->list_loaded -= 1;
+		goto end_r;
 	}
 
 	CAMEL_SERVICE_LOCK (store, connect_lock);
-/*	if ((groupwise_store->list_loaded == FALSE) && check_for_connection((CamelService *)store, ex)) {*/
-	if (check_for_connection((CamelService *)store, ex)) {
+	if ((groupwise_store->list_loaded == 1) && check_for_connection((CamelService *)store, ex)) {
 		if (!priv->cnc) {
 			if (groupwise_connect ((CamelService *)store, ex)) {
 				g_warning ("Could connect!!!\n");
@@ -1054,7 +1162,6 @@ groupwise_get_folder_info (CamelStore *store, const char *top, guint32 flags, Ca
 				g_warning ("Could not connect..failure connecting\n");
 		}
 		if (camel_groupwise_store_connected ((CamelGroupwiseStore *)store, ex)) {
-		/*	groupwise_store->list_loaded = TRUE;*/
 			if (groupwise_store->current_folder)
 				CAMEL_FOLDER_CLASS (CAMEL_OBJECT_GET_CLASS (groupwise_store->current_folder))->sync(groupwise_store->current_folder, FALSE, ex);
 			groupwise_folders_sync (groupwise_store, ex);
@@ -1069,6 +1176,7 @@ groupwise_get_folder_info (CamelStore *store, const char *top, guint32 flags, Ca
 	CAMEL_SERVICE_UNLOCK (store, connect_lock);
 
 	/*camel_exception_clear (ex);*/
+end_r:
 	info = groupwise_get_folder_info_offline (store, top, flags, ex);
 	return info;
 }
@@ -1330,9 +1438,10 @@ gboolean
 camel_groupwise_store_connected (CamelGroupwiseStore *store, CamelException *ex)
 {
 	if (((CamelOfflineStore *) store)->state == CAMEL_OFFLINE_STORE_NETWORK_AVAIL
-	    && camel_service_connect ((CamelService *)store, ex))
+	    && camel_service_connect ((CamelService *)store, ex)) {
 		return TRUE;
-
+	}
+	/*Not online, so return FALSE*/
 	return FALSE;
 }
 
