@@ -125,6 +125,7 @@ typedef struct {
 	char *id ;
 	char *name ;
 	char *item_reference;
+	char *contentid;
 	char *contentType ;
 	int size ;
 	char *date ;
@@ -297,6 +298,7 @@ void e_gw_item_set_source (EGwItem *item, char *source) ;
 int e_gw_item_get_trigger (EGwItem *item);
 void e_gw_item_set_trigger (EGwItem *item, int trigger);
 
+gboolean e_gw_item_has_attachment (EGwItem *item);
 
 typedef struct {
 	char *email;
